@@ -1,6 +1,7 @@
 from __future__ import annotations
 
 from collections.abc import Iterable
+import importlib.metadata
 import json
 import sys
 
@@ -10,6 +11,10 @@ import structlog
 from bowtie import _commands
 
 
+class _InvalidBowtieReport(Exception):
+    pass
+
+
 def writer(file=sys.stdout):
     return lambda **result: file.write(f"{json.dumps(result)}\n")
 
@@ -47,16 +52,41 @@ class Reporter:
             )
             for implementation in implementations
         }
-        self._write(implementations=metadata)
+        self._write(
+            implementations=metadata,
+            bowtie_version=importlib.metadata.version("bowtie-json-schema"),
+        )
 
     def will_speak(self, dialect):
-        self._log.info("Will speak dialect", dialect=dialect)
+        self._write(dialect=dialect)
 
-    def finished(self, count):
+    def finished(self, count, did_fail_fast):
         if not count:
             self._log.error("No test cases ran.")
         else:
             self._log.msg("Finished", count=count)
+        self._write(did_fail_fast=did_fail_fast)
+
+    def startup_failed(self, name):
+        self._log.exception("Startup failed!", logger_name=name)
+
+    def dialect_error(self, implementation, stderr):
+        self._log.error(
+            "Tried to start sending test cases, but got an error.",
+            logger_name=implementation.name,
+            stderr=stderr,
+        )
+
+    def no_implementations(self):
+        self._log.error("No implementations started successfully!")
+
+    def invalid_response(self, request, response, implementation, error):
+        self._log.exception(
+            "Invalid response",
+            logger_name=implementation.name,
+            exc_info=error,
+            request=request,
+        )
 
     def case_started(self, seq, case):
         return _CaseReporter.case_started(
@@ -109,10 +139,11 @@ class Count:
 
 
 @attrs.define(slots=False)
-class Summary:
+class _Summary:
 
     implementations: Iterable[str]
     _combined: dict = attrs.field(factory=dict)
+    did_fail_fast: bool = False
 
     def __attrs_post_init__(self):
         self.implementations = sorted(
@@ -173,8 +204,31 @@ class Summary:
                 count.failed_tests += 1
             seen[implementation] = result, failed
 
-    def combined(self):
-        return [(v, k) for k, v in sorted(self._combined.items())]
+    def see_maybe_fail_fast(self, did_fail_fast):
+        self.did_fail_fast = did_fail_fast
+
+    def case_results(self):
+        for seq, each in sorted(self._combined.items()):
+            case, results = each["case"], each["results"]
+            yield seq, case["description"], case["schema"], results
+
+
+@attrs.define
+class RunInfo:
+
+    bowtie_version: str
+    dialect: str
+    _implementations: dict[str, dict]
+
+    @classmethod
+    def from_header_lines(cls, first, second):
+        return cls(**first, **second)
+
+    def create_summary(self):
+        """
+        Create a summary object used to incrementally parse reports.
+        """
+        return _Summary(implementations=self._implementations.values())
 
 
 def from_input(input):
@@ -183,14 +237,16 @@ def from_input(input):
     """
 
     lines = (json.loads(line) for line in input)
-    header = next(lines)
-    summary = Summary(implementations=header["implementations"].values())
+    run_info = RunInfo.from_header_lines(next(lines), next(lines))
+    summary = run_info.create_summary()
 
     for each in lines:
         if "case" in each:
             summary.add_case_metadata(**each)
         elif "caught" in each:
             summary.see_error(**each)
+        elif "did_fail_fast" in each:
+            summary.see_maybe_fail_fast(**each)
         else:
             summary.see_results(**each)
-    return dict(summary=summary, results=summary.combined())
+    return dict(summary=summary, run_info=run_info)
