@@ -1,12 +1,14 @@
 from pathlib import Path
-from urllib.parse import urljoin
 import importlib.metadata
 import re
 
+from hyperlink import URL
+
 DOCS = Path(__file__).parent
 STATIC = DOCS / "_static"
 
-HOMEPAGE = "https://github.com/python-jsonschema/bowtie/"
+GITHUB = URL.from_text("https://github.com/")
+HOMEPAGE = GITHUB.child("bowtie-json-schema", "bowtie")
 
 project = "bowtie"
 author = "Julian Berman"
@@ -28,8 +30,10 @@ extensions = [
     "sphinx.ext.napoleon",
     "sphinx.ext.todo",
     "sphinx_click",
+    "sphinx_copybutton",
     "sphinx_json_schema_spec",
     "sphinxcontrib.spelling",
+    "sphinxext.opengraph",
 ]
 
 pygments_style = "lovelace"
@@ -61,11 +65,12 @@ intersphinx_mapping = {
 # -- Options for extlinks extension ------------------------------------------
 
 extlinks = {
-    "gh": (urljoin(HOMEPAGE, "%s"), None),
+    "gh": (str(HOMEPAGE.child("%s")), None),
+    "github": (str(GITHUB.child("%s")), None),
     "glossary": ("https://json-schema.org/learn/glossary.html#%s", None),
 }
 
-# -- Options for the linkcheck builder ------------------------------------
+# -- Options for the linkcheck builder ---------------------------------------
 
 
 def entire_domain(host):
@@ -75,6 +80,11 @@ def entire_domain(host):
 linkcheck_ignore = [
     entire_domain("codecov.io"),
     entire_domain("img.shields.io"),
-    urljoin(HOMEPAGE, "actions"),
-    urljoin(HOMEPAGE, "bowtie/workflows/CI/badge.svg"),
+    f"{GITHUB}.*#.*",
+    str(HOMEPAGE.child("actions")),
+    str(HOMEPAGE.child("bowtie/workflows/CI/badge.svg")),
 ]
+
+# -- Options for spelling extension ------------------------------------------
+
+spelling_show_suggestions = True
